#!/usr/bin/python3

import sys
import MeCab
import os
import time
from operator import itemgetter
from multiprocessing import Process, Lock, Queue, Manager

directory = os.fsencode("scripts")
num_parser_threads = 8
mecab = MeCab.Tagger()
scriptQ = Queue()
joinQ = Queue()
fullDictQ = Queue()

ranges = [
  {"from": ord(u"\u3300"), "to": ord(u"\u33ff")},         # compatibility ideographs
  {"from": ord(u"\ufe30"), "to": ord(u"\ufe4f")},         # compatibility ideographs
  {"from": ord(u"\uf900"), "to": ord(u"\ufaff")},         # compatibility ideographs
  {"from": ord(u"\U0002F800"), "to": ord(u"\U0002fa1f")}, # compatibility ideographs
  {'from': ord(u'\u3040'), 'to': ord(u'\u309f')},         # Japanese Hiragana
  {"from": ord(u"\u30a0"), "to": ord(u"\u30ff")},         # Japanese Katakana
  {"from": ord(u"\u2e80"), "to": ord(u"\u2eff")},         # cjk radicals supplement
  {"from": ord(u"\u4e00"), "to": ord(u"\u9fff")},
  {"from": ord(u"\u3400"), "to": ord(u"\u4dbf")},
  {"from": ord(u"\U00020000"), "to": ord(u"\U0002a6df")},
  {"from": ord(u"\U0002a700"), "to": ord(u"\U0002b73f")},
  {"from": ord(u"\U0002b740"), "to": ord(u"\U0002b81f")},
  {"from": ord(u"\U0002b820"), "to": ord(u"\U0002ceaf")}  # included as of Unicode 8.0
]

def is_cjk(char):
  return any([range["from"] <= ord(char) <= range["to"] for range in ranges])


def buildScriptQueue(scriptQ): #puts file paths to be read by parser threads in a queue
    for script in os.listdir(directory):
        scriptQ.put(script)

def parseScript(script, tempDict):
    filename = os.fsdecode(script)
    file = open(os.path.join(os.fsdecode(directory),filename), "r")
    parsed = mecab.parse(file.read())
    for line in parsed.splitlines():
        parts = line.split('	')
        if len(parts) > 1:
            word = parts[1].split(",")[6] #get the dictionary form of the word
            updateDict(word, tempDict)

def updateDict(word, dictionary):
    if word in dictionary:
        dictionary[word] += 1
    else:
        dictionary[word] = 1

def parseJob(): #processes that read, parse and build dictionaries for individual scripts
    while True:
        script = scriptQ.get()
        if script is None:
            break
        tempDict = dict()
        parseScript(script, tempDict)
        joinQ.put(tempDict)
    while not joinQ.empty():
        time.sleep(0.1)

def mergeJob(): #process that joins the generated dictionaries from parseJobs together
    fullDict = dict()
    while True:
        tempDict = joinQ.get()
        if tempDict is None:
            break
        mergedicts(fullDict, tempDict)
    fullDictQ.put(fullDict)
    while not fullDictQ.empty():
        time.sleep(0.1)
    
def mergedicts(full, temp):
    for key in temp:
        if key in full:
            full[key] += temp[key]
        else:
            full[key] = temp[key]

def generateDict():
    buildScriptQueue(scriptQ)

    parseProcesses = []
    for i in range(num_parser_threads):
        scriptQ.put(None)
        p = Process(target=parseJob)
        p.start()
        parseProcesses.append(p)

    mergep = Process(target=mergeJob)
    mergep.start()

    for p in parseProcesses:
        p.join()
        
    joinQ.put(None)
    return fullDictQ.get()


dictionary = generateDict()


trimlist = [] #remove some garbage from the results
for key in dictionary:
    if len(key) < 4: #delete kana only words with less than 4 characters
        deletionCandidate = True
        for letter in key:
            if not ord(u'\u3040') <= ord(letter) <= ord(u"\u30ff"):
                deletionCandidate = False
        if deletionCandidate:
            trimlist.append(key)
    for letter in key: #delete random garbage characters
        if not is_cjk(letter):
            trimlist.append(key)
            
for key in trimlist:
    if key in dictionary:
        dictionary.pop(key)

tupleList = list(dictionary.items())
tupleList.sort(key=itemgetter(1), reverse=True) #list of (word, #occurrences) tuples, ordered by occurrences

sortedwords = sorted(dictionary, key=dictionary.get, reverse=True) #list of sorted words
